# coding: utf-8
from __future__ import unicode_literals
import re
import json
from ..extractor.nbc import NBCIE as Old
from ..extractor.common import InfoExtractor
from ..extractor.theplatform import ThePlatformIE
from ..utils import (
    smuggle_url,
    update_url_query,
    int_or_none,
    urlencode_postdata
)

class NBCIE(Old):
    def _real_extract(self, url):
        try:
            result = super(NBCIE, self)._real_extract(url)
            ie = ThePlatformIE()
            ie.set_downloader(self._downloader)
            testResult = ie._real_extract(result['url'])
            return result
        except:
            permalink, video_id = re.match(self._VALID_URL, url).groups()
            webpage = self._download_webpage(url, url).replace('https://schema.org', 'http://schema.org')
            video_data = self._search_json_ld(webpage, '', fatal=False)

            if video_id:
                try:
                    token = self._get_cookies(url).get('park-fr').value
                    mpxAccountId = self._search_regex(r'media/guid/(\d+)/%s'%video_id, webpage, '')

                    headers = {
                        "accept":"application/access-v1+json",
                       "Content-Type": "application/json",
                       "authorization": "NBC-Referrer key=desktop_nbcuniversal,idmToken=%s,version=3.0,time=2019-08-20T08:40:05.978Z" % token,
                       "referrer": url,
                    }

                    data = json.dumps({'device': 'web','mpxAccountId': mpxAccountId}).encode()


                    obj =self._download_json('https://access-cloudpath.media.nbcuni.com/access/vod/nbcuniversal/%s' % video_id, video_id,
                                        data = data, headers = headers)
                    linktheplatformURL = obj['url'] + '&manifest=m3u&autoPlay=true&policy=147097224&fallbackSiteSectionId=9244655&siteSectionId=oneapp_desktop_computer_web_ondemand&switch=HLSOriginSecure&MVPDid=undefined&_fw_vcid2=&_fw_h_referer=www.nbc.com&schema=2.0&params=&format=SMIL&tracking=true&formats=M3U+appleHlsEncryption,M3U+none,MPEG4,MP3'
                    formats = []
                    formats.extend(self._extract_smil_formats(linktheplatformURL, video_id))

                    return {
                        '_type': 'video',
                        'id': video_id,
                        'title': video_data.get('title'),
                        'formats': formats,
                        'description': video_data.get('description'),
                        'keywords': video_data.get('keywords'),
                        'season_number': int_or_none(video_data.get('seasonNumber')),
                        'episode_number': int_or_none(video_data.get('episodeNumber')),
                        'series': video_data.get('showName'),
                    }
                except:
                    query = {
                        'mbr': 'true',
                        'manifest': 'm3u',
                    }
                    theplatform_url = smuggle_url(update_url_query(
                        'http://link.theplatform.com/s/NnzsPC/media/guid/2410887629/' + video_id,
                        query), {'force_smil_url': True})
                    return {
                        '_type': 'url_transparent',
                        'id': video_id,
                        'title': video_data.get('title'),
                        'url': theplatform_url,
                        'description': video_data.get('description'),
                        'keywords': video_data.get('keywords'),
                        'season_number': int_or_none(video_data.get('seasonNumber')),
                        'episode_number': int_or_none(video_data.get('episodeNumber')),
                        'series': video_data.get('showName'),
                        'ie_key': 'ThePlatform',
                    }


class NBCNewsVideoIE(InfoExtractor):
    _VALID_URL = r'(?x)https?://(?:www\.)?nbcnews\.com/video/*(?:.*-)?(?P<id>[^/?]+)'
    def _real_extract(self, url):
        video_id = self._match_id(url)
        data = self._download_json('https://www.nbcnews.com/bentoapi/videos/' + video_id, '')
        videoAsset = data['data']['mmvo' + video_id]['videoAssets'][0]
        link = videoAsset['publicUrl']

        return {
            '_type': 'url_transparent',
            'id': video_id,
            'url': link,            
            'ie_key': 'ThePlatform',
        }

